/*
 * SHA1
 * 
 * Implementation of the SHA-1 defined in FIPS 180-1.
 * Copyright-free 2004-2014 by Masahiko Nakamura.
 * 
 * ver 1.0  2014/08/14  64bit support
 * ver 0.1  2004/03/27  alpha version
 */

#include "sha1.h"

typedef	unsigned int	WORD32;

#define	S(n, X)	(((X) << (n)) | ((X) >> (32 - (n))))

#define	F0(B, C, D)	(((B) & (C)) | ((~(B)) & (D)))
#define	F2(B, C, D)	((B) ^ (C) ^ (D))
#define	F4(B, C, D)	(((B) & (C)) | ((B) & (D)) | ((C) & (D)))
#define	F6(B, C, D)	((B) ^ (C) ^ (D))

#define	K0	(0x5A827999)
#define	K2	(0x6ED9EBA1)
#define	K4	(0x8F1BBCDC)
#define	K6	(0xCA62C1D6)

#define	H0	(0x67452301)
#define	H1	(0xEFCDAB89)
#define	H2	(0x98BADCFE)
#define	H3	(0x10325476)
#define	H4	(0xC3D2E1F0)

WORD32 h[5];
unsigned char buff[64];
int bp;
WORD32 len_lo, len_hi;

/* initialize */
void sha1init(void)
{
	h[0] = H0;
	h[1] = H1;
	h[2] = H2;
	h[3] = H3;
	h[4] = H4;
	
	bp = 0;
	
	len_lo = 0;
	len_hi = 0;
	
	return;
}

/* proceed block */
void sha1proc(unsigned char *block)
{
	WORD32 a, b, c, d, e;
	WORD32 w[80];
	WORD32 temp;
	int t;
	
	// Step1
	for (t = 0; t <= 15; t++) {
		w[t] = (block[t * 4] << 24) + (block[t * 4 + 1] << 16)
			 + (block[t * 4 + 2] << 8) + (block[t * 4 + 3]);
	}
	// Step2
	for (t = 16; t <= 79; t++) {
		temp = w[t - 3] ^ w[t - 8] ^ w[t - 14] ^ w[t - 16];
		w[t] = S(1, temp);
	}
	// Step3
	a = h[0];
	b = h[1];
	c = h[2];
	d = h[3];
	e = h[4];
	// Step4
	for (t = 0; t <= 19; t++) {
		temp = S(5, a) + F0(b, c, d) + e + w[t] + K0;
		e = d;
		d = c;
		c = S(30, b);
		b = a;
		a = temp;
	}
	for (t = 20; t <= 39; t++) {
		temp = S(5, a) + F2(b, c, d) + e + w[t] + K2;
		e = d;
		d = c;
		c = S(30, b);
		b = a;
		a = temp;
	}
	for (t = 40; t <= 59; t++) {
		temp = S(5, a) + F4(b, c, d) + e + w[t] + K4;
		e = d;
		d = c;
		c = S(30, b);
		b = a;
		a = temp;
	}
	for (t = 60; t <= 79; t++) {
		temp = S(5, a) + F6(b, c, d) + e + w[t] + K6;
		e = d;
		d = c;
		c = S(30, b);
		b = a;
		a = temp;
	}
	// Step5
	h[0] += a;
	h[1] += b;
	h[2] += c;
	h[3] += d;
	h[4] += e;
	
	return;
}

/* add data */
void sha1data(unsigned char *data, int len)
{
	int i;
	
	for (i = 0; i < len; i++) {
		buff[bp] = data[i];
		bp++;
		len_lo += 8;
		if (len_lo == 0) {
			len_hi++;
		}
		if (bp == 64) {
			sha1proc(buff);
			bp = 0;
		}
	}
	
	return;
}

/* get hash */
void sha1hash(unsigned char *hash)
{
	int i;
	
	buff[bp] = 0x80;
	bp++;
	for (i = bp; i < 64; i++) {
		buff[i] = 0;
	}
	if (bp > 56) {
		sha1proc(buff);
		bp = 0;
		for (i = bp; i < 64; i++) {
			buff[i] = 0;
		}
	}
	buff[56] = (unsigned char)((len_hi & 0xff000000) >> 24);
	buff[57] = (unsigned char)((len_hi & 0x00ff0000) >> 16);
	buff[58] = (unsigned char)((len_hi & 0x0000ff00) >> 8);
	buff[59] = (unsigned char)(len_hi & 0x000000ff);
	buff[60] = (unsigned char)((len_lo & 0xff000000) >> 24);
	buff[61] = (unsigned char)((len_lo & 0x00ff0000) >> 16);
	buff[62] = (unsigned char)((len_lo & 0x0000ff00) >> 8);
	buff[63] = (unsigned char)(len_lo & 0x000000ff);
	sha1proc(buff);
	bp = 0;
	
	for (i = 0; i < 5; i++) {
		hash[i * 4] = (unsigned char)((h[i] & 0xff000000) >> 24);
		hash[i * 4 + 1] = (unsigned char)((h[i] & 0x00ff0000) >> 16);
		hash[i * 4 + 2] = (unsigned char)((h[i] & 0x0000ff00) >> 8);
		hash[i * 4 + 3] = (unsigned char)(h[i] & 0x000000ff);
	}
	
	return;
}

